     //////////////////////////////////////////////
    // THE COMPARATIVE CODE-UNDERSTANDING SYSTEM //
   // Written by Andrew Broad                   ///
  //                                           ////
 // CCUS.java (2002:11:20 16:18)              /////
///////////////////////////////////////////////////
// Methods of CCUS:                          //////
// - static main                             //////
// - static usage                            /////
// - static processArguments                 ////
// - static processFlags                     ///
// - time                                    //
//////////////////////////////////////////////

import java.util.*;
import java.io.*;

import jex.*;

import jwnl.data.*;
import jwnl.data.list.pointer_target.*;
import jwnl.data.relationship.*;
import jwnl.dictionary.DictionaryDatabase;
import jwnl.dictionary.hashtable.HashtableBackedDictionary;

/*****************************************************************************
** The class `CCUS' is the top-level class of the Comparative Constraint-
** Understanding System. It is not meant to be instantiated.
*****************************************************************************/

public class CCUS {
	public static boolean _nameThesaurus= false;
	public static boolean _frameClassThesaurus= false;
	public static DictionaryDatabase _db= null;

	public static boolean _semanticCases= true;
	public static boolean _subexpressionCases= true;
	public static boolean _extractHLCs= true;
	public static boolean _considerExtractionPromptsTogether= false;
	public static boolean _rememberHLCs= true;
	public static boolean _monogamy= false;
	public static boolean _reportNonDifferences= false;
	public static boolean _generateHTML= false;
	public static boolean _generateIndexHTML= false;
	public static boolean _generateSummary= false;
	public static boolean _onlyInputModelFramesInHTML= false;
	public static boolean _showTimes= false;

	private static String _expressFilename1= null;
	private static String _expressFilename2= null;
	private static String _caseLibraryFilename
	       = "/home/broada/jws/PhD_CUS/CASES/library.cas";
	private static boolean _parseFirst= false;

	/********************************************************************/
	// static CCUS.main : String[] -> void
	/*********************************************************************
	** The main entry point to the Comparative Constraint-Understanding
	** System.
	** @param argv The vector of command line arguments. There should be
	**  three of these: the filenames of two EXPRESS models (.exp) to
	**  parse using jex, and a case library (.cas). If the latter is not
	**  specified, it defaults to "CASES/library.cas". There can also be
	**  command-line flags:
	**  -s disables semantic equivalence/strength cases
	**  -u disables subexpression cases
	**  -h disables the extraction of HLCs
	**  -x causes the HLC-CCUS to consider extraction prompts separately
	**  -r no remembrance
	**  -m monogamy
	**  -p pretty-prints HLCs in the CCUS output
	**  -n causes non-differences to be output
	**  -i causes only input model frames to be included in the HTML
	**  -t prints timestamps at the major phase boundaries
	**  -g causes HTML files for frames to be generated without asking
	**  -w generates index.html for this experiment
	**  -y generates summary.html for this experiment
	*********************************************************************/

	public static void main (String argv[]) {
		String name= null;
		Frame frameNetwork1= null, frameNetwork2= null;
		long startTimestamp= System.currentTimeMillis();
		long parseTimestamp= 0, parseTime= 0,
		     loadCLTimestamp= 0, loadCLTime= 0,
		     loadParseTimestamp= 0,
		     framesTimestamp= 0, parseFramesTime= 0,
		     loadThesaurusTimestamp= 0, loadThesaurusTime= 0,
		     comparisonTimestamp= 0, comparisonTime= 0;

		int numberOfFrames1= -1, numberOfFrames2= -1;
		try {
			System.setProperty("java.compiler","none");
			System.setProperty("jex.licenseLocation",
			                   "27000@manatee");

			processArguments(argv);

			if (_generateIndexHTML || _generateSummary) {
				System.out.print("Name this experiment: ");
				BufferedReader input= new BufferedReader
				                        (new InputStreamReader
				                             (System.in));
				name= input.readLine();
				System.out.println();
			}
			Frame._markFramesAsInputModel= true;

			Model inputModel1= null;
			Model inputModel2= null;

			if (_parseFirst) {
				System.out.println(time() + "Parsing EXPRESS "
				+ "model in \"" + _expressFilename1 + "\" usi"
				+ "ng Jex...");
				inputModel1= (Model)Model.readText
				             (new File(_expressFilename1));
				System.out.println
				 ("Done parsing first EXPRESS model.\n");

				System.out.println(time() + "Parsing EXPRESS "
				+ "model in \"" + _expressFilename2 + "\" usi"
				+ "ng Jex...");
				inputModel2= (Model)Model.readText
				             (new File(_expressFilename2));
				System.out.println
				 ("Done parsing second EXPRESS model.\n");

				parseTimestamp= System.currentTimeMillis();
				parseTime= parseTimestamp - startTimestamp;

				System.out.println(time() + "Loading case lib"
				+ "rary from \"" + _caseLibraryFilename + "\""
				+ "...");
				CaseLibrary.load(_caseLibraryFilename);
				System.out.println
				 ("Done loading case library.\n");

				loadParseTimestamp=System.currentTimeMillis();
				loadCLTime= loadParseTimestamp-parseTimestamp;
			} else {
				System.out.println(time() + "Loading case lib"
				+ "rary from \"" + _caseLibraryFilename + "\""
				+ "...");
				CaseLibrary.load(_caseLibraryFilename);
				System.out.println
				("Done loading case library.\n");

				loadCLTimestamp= System.currentTimeMillis();
				loadCLTime= loadCLTimestamp - startTimestamp;

				System.out.println(time() + "Parsing EXPRESS "
				+ "model in \"" + _expressFilename1 + "\" usi"
				+ "ng Jex...");
				inputModel1= (Model)Model.readText
				             (new File(_expressFilename1));
				System.out.println
				 ("Done parsing first EXPRESS model.\n");

				System.out.println(time() + "Parsing EXPRESS "
				+ "model in \"" + _expressFilename2 + "\" usi"
				+ "ng Jex...");
				inputModel2= (Model)Model.readText
				             (new File(_expressFilename2));
				System.out.println
				 ("Done parsing second EXPRESS model.\n");

				loadParseTimestamp=System.currentTimeMillis();
				parseTime= loadParseTimestamp-loadCLTimestamp;
			}

			System.out.println(time() + "Converting Jex objects "
			+ "to frames (first model)...");
			Frame._numberOfFrames= 0;
			frameNetwork1= JexToFrames.convert(inputModel1,false);
			numberOfFrames1= Frame._numberOfFrames;
			System.out.println("Done converting Jex objects to "
			                 + "frames (first model).\n");

			System.out.println(time() + "Converting Jex objects "
			+ "to frames (second model)...");
			Frame._numberOfFrames= 0;
	 		frameNetwork2= JexToFrames.convert(inputModel2,false);
			numberOfFrames2= Frame._numberOfFrames;
	 		System.out.println("Done converting Jex objects to "
	 		                 + "frames (second model).\n");

			framesTimestamp= System.currentTimeMillis();
			parseFramesTime= framesTimestamp - loadParseTimestamp
			                 + parseTime;

			if (System.getProperty("WNHOME") != null
			 && System.getProperty("WNSEARCHDIR") != null) {
				System.out.println(time() + "Loading thesaurus...");
				_db= new HashtableBackedDictionary();
				_nameThesaurus= _db.synsets(POS.NOUN).
				                    hasMoreElements();
				System.out.println("\nName thesaurus " +
				 (_nameThesaurus ? "enabled." : "disabled."));
				_frameClassThesaurus= _db.synsets(POS.ADVERB).
				                          hasMoreElements();
				System.out.println("Frame-class thesaurus " +
				 (_frameClassThesaurus
				  ? "enabled." : "disabled."));
				System.out.println("\nDone loading thesaurus."
				                 + "\n");
			}

			loadThesaurusTimestamp = System.currentTimeMillis();
			loadThesaurusTime= loadThesaurusTimestamp
			                   - framesTimestamp;

			System.out.println(
				"----------------------------------------" +
				"----------------------------------------");
			System.out.println(time() + "Comparing models...\n");

			ComparisonEngine.compareModels
			                 (frameNetwork1, frameNetwork2);

			comparisonTimestamp= System.currentTimeMillis();
			comparisonTime= comparisonTimestamp
			                - loadThesaurusTimestamp;

		} catch (Throwable t) {
			System.err.println(
				"----------------------------------------" +
				"----------------------------------------");
			System.err.println(time() + "A Throwable of type "
			+ t.getClass() + " has been caught in "
			+ "CCUS.main!\n\ttoString == " + t + "\n");
			//System.err.println("Its Message is:-\n"
			//                  + t.getMessage() + "\n");
			//System.err.println("Its LocalizedMessage is:-\n"
			//+ t.getLocalizedMessage());
			if (! (t instanceof Error)) {
				System.out.print(time() + "Print stack trace?"
				+ " (y/n)? ");
				BufferedReader input= new BufferedReader
				           (new InputStreamReader(System.in));
				String printStackTrace= null;
				try {
					printStackTrace= input.readLine();
				} catch (IOException ioe) {
					System.err.println("CCUS.main: IOExce"
					+ "ption thrown by BufferedReader.rea"
					+ "dLine()");
					System.exit(1);
				}
				System.out.println();
				if (printStackTrace.
				    regionMatches(true,0,"y",0,1))
				{
					t.printStackTrace();
				}
			} else {
				System.err.println("Not printing stack trace "
				+ "because " + t + " instanceof Error");
			}
		}
		System.out.println(
			"----------------------------------------" +
			"----------------------------------------");
		///////////////////////////////////
		// Print experimental statistics //
		///////////////////////////////////
		System.out.println("Number of frames in  first model: "
		+ numberOfFrames1);
		System.out.println("Number of frames in second model: "
		+ numberOfFrames2);
		System.out.println("Number of HLCs extracted from  first "
		+ "model: " + HLC_CCUS._numberOfHLCsExtracted1);
		System.out.println("Number of HLCs extracted from second "
		+ "model: " + HLC_CCUS._numberOfHLCsExtracted2);
		System.out.println("Number of     comparisons: "
		+ CCUS_FMA._numberOfComparisons);
		System.out.println("Number of HLC-comparisons: "
		+ CCUS_FMA._numberOfHLCComparisons);
		System.out.println("Number of     correspondences: "
		+ Correspondences._numberOfCorrespondences);
		System.out.println("Number of HLC-correspondences: "
		+ Correspondences._numberOfHLCCorrespondences);
		System.out.println("Number of differences: "
		+ CCUS_FMA._numberOfDifferences);
		System.out.println("Number of frame-class-thesaurus-lookups: "
		+ CorrespondenceHeuristics._frameClassThesaurusLookups);
		System.out.println("Number of name-thesaurus-lookups: "
		+ CorrespondenceHeuristics._nameThesaurusLookups);
		Enumeration frameClasses=
		  Correspondences._numberOfCorrespondencesHashtable.keys();
		while (frameClasses.hasMoreElements()) {
			FrameClass fc= (FrameClass)frameClasses.nextElement();
			System.out.println("=Number of " + fc
			+ "-correspondences: "
			+ Correspondences._numberOfCorrespondencesHashtable.
			get(fc));
		}
		if (_showTimes) {
			System.out.println("Time to load case library: "
			+ ((double)loadCLTime)/1000 + "s");
			System.out.println("Time to parse models and convert "
			+ "to frames: " + ((double)parseFramesTime/1000)+"s");
			System.out.println("Time to load thesaurus: "
			+ ((double)loadThesaurusTime)/1000 + "s");
			System.out.println("Time to compare models: "
			+ ((double)comparisonTime)/1000 + "s");
		}
		System.out.println(
			"----------------------------------------" +
			"----------------------------------------");
		if (! _generateHTML) {
			System.out.print(time() + "Generate HTML for the fram"
			+ "es (y/n)? ");
			BufferedReader input= new BufferedReader
			                   (new InputStreamReader(System.in));
			String generateHTML= null;
			try {
				generateHTML= input.readLine();
			} catch (IOException ioe) {
				System.err.println("CCUS.main: IOException th"
				+ "rown by BufferedReader.readLine()");
				System.exit(1);
			}
			System.out.println();
			_generateHTML= generateHTML.
			               regionMatches(true,0,"y",0,1);
		}
		if (_generateHTML) {
			System.out.println("\n" + time() + "Generating HTML "
			+ "in /usr/local/tmp/users/broada/FRAMES/...\n");

			PrintWriter indexHTML= null;

			if (_generateIndexHTML) {
				System.out.println("Generating ./index.html");
				try {
					FileOutputStream fo= new
					  FileOutputStream("index.html");
					indexHTML= new PrintWriter(fo,true);
					IndexHTML.generateHead(indexHTML,name,
				 	_expressFilename1, _expressFilename2);
				} catch (FileNotFoundException fnfe) {
					System.err.println("CCUS.main: FileNo"
					+ "tFoundException while generating "
					+ "index.html");
					System.exit(1);
				}
			}
			Frame.generateFIL(true, _onlyInputModelFramesInHTML,
			                  indexHTML);
			if (_generateIndexHTML) {
				IndexHTML.generateTail(indexHTML,name);
			}

			if (_generateSummary) {
				System.out.println("Generating ./"
				                 + "summary.html");
				try {
					FileOutputStream fo= new
					  FileOutputStream("summary.html");
					PrintWriter summFile
					= new PrintWriter(fo,true);
					Summary.generateFile(summFile, name,
					        frameNetwork1, frameNetwork2);
				} catch (FileNotFoundException fnfe) {
					System.err.println("CCUS.main: FileNo"
					+ "tFoundException while generating "
					+ "summary.html");
					System.exit(1);
				}
			}
		}
		if (_showTimes) {
			System.out.println("\n" + time() + "Finished.");
		}
	}

	/********************************************************************/
	// static CCUS.usage : void -> void
	/*********************************************************************
	** Print the usage report when the command-line arguments are invalid,
	** and exit the system.
	*********************************************************************/

	private static void usage () {
		System.err.println("Usage: Three arguments: two EXPRESS model"
		+ "s (.exp files) and a case library (.cas file). The latter "
		+ "is optional - if omitted, the default is \"CASES/library.c"
		+ "as\"");
		System.err.println("    Flags:-");
		System.err.println("-s: disables semantic equivalence/"
		+ "strength cases");
		System.err.println("-u: disables subexpression cases");
		System.err.println("-h: disables the extraction of HLCs");
		System.err.println("-x: consider extraction prompts "
		+ "separately");
		System.err.println("-r: no remembrance");
		System.err.println("-m: monogamy");
		System.err.println("-p: pretty-prints HLCs in the CCUS output"
		);
		System.err.println("-n: report non-differences");
		System.err.println("-i: include only input model frames in "
		+ "HTML");
		System.err.println("-t: show times at phase boundaries");
		System.err.println("-g: generate HTML without asking");
		System.err.println("-w: generate index.html");
		System.err.println("-y: generate summary.html");
		System.exit(1);
	}

	/********************************************************************/
	// static CCUS.processArguments : String[] -> void
	/*********************************************************************
	** A subroutine to process the command-line arguments to the CCUS.
	** @param argv The vector of command-line arguments. There should be
	**  three of these: the filenames of two EXPRESS models (.exp) to
	**  parse using jex, and a case library (.cas). If the latter is not
	**  specified, it defaults to "CASES/library.cas".
	*********************************************************************/

	private static void processArguments (String argv[]) {
		for (int i= 0; i < argv.length; i++) {
			if (argv[i].endsWith(".exp")
			 && _expressFilename1 == null)
			{
				_expressFilename1= argv[i];
			}
			if (argv[i].endsWith(".exp")
			 && _expressFilename1 != null)
			{
				_expressFilename2= argv[i];
				_parseFirst= false;
			}
			if (argv[i].endsWith(".cas")) {
				_caseLibraryFilename= argv[i];
				_parseFirst= true;
			}
			if (argv[i].startsWith("-")) {
				processFlags(argv[i]);
			}
		}
		ExtractionEngine._showIntermediateOutput= false;

		if (_expressFilename1 == null || _expressFilename2 == null) {
			usage();
		}
	}

	/********************************************************************/
	// static CCUS.processFlags : String[] -> void
	/*********************************************************************
	** A subroutine to process the flags in a command-line argument
	** starting with `-'. This method assumes that it is only called with
	** strings starting with `-', and ignores the first character.
	** @param arg A `flags' command-line argument (i.e. `-' followed by
	**            letters).
	*********************************************************************/

	private static void processFlags (String arg) {
		char[] flags= arg.toCharArray();

		for (int i= 1; i < flags.length; i++) {
			switch (flags[i]) {
				case 's':
					_semanticCases= false;
					break;
				case 'u':
					_subexpressionCases= false;
					break;
				case 'h':
					_extractHLCs= false;
					break;
				case 'r':
					_rememberHLCs= false;
					break;
				case 'm':
					_monogamy= true;
					break;
				case 'x':
					_considerExtractionPromptsTogether
					= true; break;
				case 'p':
					ExtractionEngine.
					  _prettyPrintHLCs= true;
					break;
				case 'n':
					_reportNonDifferences= true;
					break;
				case 'g':
					_generateHTML= true;
					break;
				case 'w':
					_generateIndexHTML=
					_generateHTML= true;
					break;
				case 'y':
					_generateSummary=
					_generateHTML= true;
					break;
				case 'i':
					_onlyInputModelFramesInHTML= true;
					break;
				case 't':
					_showTimes= true;
					break;
				default:
					System.err.println("Warning: -"
					+ flags[i] + " not recognised.");
			}
		}
	}

	/********************************************************************/
	// static CCUS.time : void -> String
	/*********************************************************************
	** If _showTimes is true, return the current time as a string of the
	** form "(HH:MM:SS.SSS) ", otherwise return the empty string.
	** @return The time string.
	*********************************************************************/

	private static String time () {
		String time= "";
		if (_showTimes) {
			GregorianCalendar gregorc= new GregorianCalendar();
			//////////////
			// hours... //
			//////////////
			time= "(";
			int hours= gregorc.get(GregorianCalendar.HOUR_OF_DAY);
			if (hours < 10) time+= "0";
			time+= hours + ":";
			////////////////
			// minutes... //
			////////////////
			int minutes= gregorc.get(GregorianCalendar.MINUTE);
			if (minutes < 10) time+= "0";
			time+= minutes + ":";
			////////////////
			// seconds... //
			////////////////
			int seconds= gregorc.get(GregorianCalendar.SECOND);
			if (seconds < 10) time+= "0";
			time+= seconds + ".";
			/////////////////////
			// milliseconds... //
			/////////////////////
			int ms= gregorc.get(GregorianCalendar.MILLISECOND);
			if (ms < 100) {
				time+= "0";
				if (ms < 10) time+= "0";
			}
			time+= ms + ") ";
		}
		return time;
	}
}
