      ////////////////////////////////////////
     // THE CONSTRAINT-UNDERSTANDING SYSTEM //
    //  (Version 2)                        ///
   // Written by Andrew Broad             ////
  //                                     /////
 // IndexHTML.java (2002:07:22 13:52)   /////
////////////////////////////////////////////
// Methods of IndexHTML:               ////
// - generateHead                      ///
// - generateTail                      //
////////////////////////////////////////

import java.io.*;
import java.util.*;

/*****************************************************************************
** The class `IndexHTML' synthesises the file "index.html" for a CCUS
** experiment on my website.
*****************************************************************************/

public class IndexHTML implements Serializable {
	/********************************************************************/
	// static IndexHTML.generateHead : PrintWriter x String
	//                                 x String x String
	//                                 -> void
	/*********************************************************************
	** Generates the head of index.html, plugging in the two models.
	** @param indexHTML The PrintWriter object for index.html.
	** @param name The name of the experiment.
	** @param model1 Filename of the first model.
	** @param model2 Filename of the second model.
	*********************************************************************/

	public static void generateHead (PrintWriter indexHTML, String name,
	                                 String model1, String model2)
	{
		indexHTML.println("<HTML><TITLE>" + name + "</TITLE>");
		indexHTML.println("<H1><A HREF=\"../../../\">Andrew Broad's "
		+ "Website</A><BR>");
		indexHTML.println("<A HREF=\"../../\">Computer Science</A>"
		+ "<BR>");
		indexHTML.println("<A HREF=\"../\">PhD Project</A><BR>");
		indexHTML.println("<A HREF=\"../results.html\">Experimental "
		+ "Results</A><BR>");
		indexHTML.println(name + "</H1>");
		indexHTML.println();
		indexHTML.println("<HR>");
		indexHTML.println("<H3>First Model</H3>");
		indexHTML.println("<PRE>");
		BufferedReader in= null; FileInputStream fis= null;
		try {
			in= new BufferedReader(new InputStreamReader
			         (fis= new FileInputStream(model1)));
		} catch (FileNotFoundException fnfe) {
			System.err.println("IndexHTML.generateHead: FileNot"
			+ "FoundException for \"" + model1 + "\"");
			System.exit(1);
		}
		String line;
		try {
			while ((line= in.readLine()) != null) {
				indexHTML.println(line);
			}
			fis.close();
		} catch (IOException ioe) {
			System.err.println("IndexHTML.generateHead: IOExcepti"
			+ "on thrown by BufferedReader.readLine()");
			System.exit(1);
		}
		indexHTML.println("</PRE>");
		indexHTML.println();
		indexHTML.println("<H3>Second Model</H3>");
		indexHTML.println("<PRE>");
		try {
			in= new BufferedReader(new InputStreamReader
			         (fis= new FileInputStream(model2)));
		} catch (FileNotFoundException fnfe) {
			System.err.println("IndexHTML.generateHead: FileNot"
			+ "FoundException for \"" + model2 + "\"");
			System.exit(1);
		}
		try {
			while ((line= in.readLine()) != null) {
				indexHTML.println(line);
			}
			fis.close();
		} catch (IOException ioe) {
			System.err.println("IndexHTML.generateHead: IOExcepti"
			+ "on thrown by BufferedReader.readLine()");
			System.exit(1);
		}
		indexHTML.println("</PRE>");
		indexHTML.println();
		indexHTML.println("<H3>Experimental Results</H3>");
		indexHTML.println();
		indexHTML.println("<H3>Output (Mode E)</H3>");
		indexHTML.println();
		indexHTML.println("<UL>");
		indexHTML.println("<LI><A HREF=\"out.txt\">CCUS output "
		+ "(execution trace)</A></LI>");
		indexHTML.println("<LI><A HREF=\"summary.html\">Summarised "
		+ "CCUS output</A></LI>");
		indexHTML.println("</UL>");
		indexHTML.println();
		indexHTML.println("<P>The frames generated by the CCUS (and "
		+ "output as HTML-ised FIL files) follow - a file for each "
		+ "frame-class:");
		indexHTML.println("<UL>");
	}

	/********************************************************************/
	// static IndexHTML.generateTail : PrintWriter x String
	//                                 -> void
	/*********************************************************************
	** Generates the tail of index.html.
	** @param indexHTML The PrintWriter object for index.html.
	** @param name The name of the experiment.
	*********************************************************************/

	public static void generateTail (PrintWriter indexHTML, String name) {
		indexHTML.println("</UL>");
		indexHTML.println("HLCs:");
		indexHTML.println("<UL>");
		indexHTML.println("</UL></P>");
		indexHTML.println("<HR>");
		indexHTML.println("<A HREF=\"../../../email.html\">"
		+ "Email me</A>");
		indexHTML.println("</HTML>");
	}
}
