      ////////////////////////////////////////
     // THE CONSTRAINT-UNDERSTANDING SYSTEM //
    //  (Version 2)                        ///
   // Written by Andrew Broad             ////
  //                                     /////
 // Logical.java (2001:03:05 12:26)     //////
//////////////////////////////////////////////
// Methods of Logical:                 //////
// - Logical constructor               /////
// - equals                            ////
// - toString                          ///
// - FILise                            //
////////////////////////////////////////

import java.io.*;

/*****************************************************************************
** The class `Logical' represents logical literals (i.e. TRUE, FALSE or
** UNKNOWN) in FIL.
*****************************************************************************/

public class Logical implements Serializable, FILisable {
	private int _value;

	public final static Logical TRUE= new Logical(1);
	public final static Logical FALSE= new Logical(-1);
	public final static Logical UNKNOWN= new Logical(0);

	/********************************************************************/
	// Logical constructor : int ->
	/*********************************************************************
	** Constructs an instance of Logical, setting its value.
	** @param value The value to set to (either TRUE, FALSE or UNKNOWN).
	*********************************************************************/

	private Logical (int value) {
		_value= value;
	}

	/********************************************************************/
	// Logical.equals : Object -> boolean
	/*********************************************************************
	** Returns true if obj is an instance of Logical, and it contains the
	** same value as this object.
	** @param obj The object to be compared with this object.
	** @return true if the objects are equal, false if they are not.
	*********************************************************************/

	public boolean equals (Object obj) {
		return (obj instanceof Logical)
		    && (_value == ((Logical)obj)._value);
	}

	/********************************************************************/
	// Logical.toString : void -> String
	/*********************************************************************
	** Returns a string representation of the Logical object.
	** @return A string representation of the Logical object.
	*********************************************************************/

	public String toString () {
		switch (_value) {
			case -1: return "false";
			case  1: return "true";
			default: return "unknown";
		}
	}

	/********************************************************************/
	// Logical.FILise : boolean -> String
	/*********************************************************************
	** Returns a string representing the logical literal in FIL.
	** @param HTML If true, the FIL will be augmented with HTML tags.
	** @return A string representing the logical literal in FIL.
	*********************************************************************/

	public String FILise (boolean HTML) {
		return toString();
	}
}
