      ///////////////////////////////////////////////
     // THE CONSTRAINT-UNDERSTANDING SYSTEM        //
    //  (Version 2)                               ///
   // Written by Andrew Broad                    ////
  //                                            /////
 // OrderedHashtable.java (2000:12:12 17:36)   //////
/////////////////////////////////////////////////////
// Methods of OrderedHashtable:                //////
// - OrderedHashtable constructor              /////
// - put                                       ////
// - remove                                    ///
// - keyListIterator                           //
////////////////////////////////////////////////

import java.io.*;
import java.util.*;

import jex.*;

/*****************************************************************************
** The class `OrderedHashtable' is a subclass of Hashtable that records the
** order of keys (by overriding the methods put and remove), and adds a method
** keyListIterator to return an iterator over the keys in the order in which
** they were put.
*****************************************************************************/

public class OrderedHashtable extends Hashtable {
	private transient Vector _keys= new Vector();

	private void writeObject (ObjectOutputStream stream)
	                        throws java.io.IOException
	{
		stream.writeObject(_keys);
		stream.defaultWriteObject();
	}

	private void readObject (ObjectInputStream stream)
	                        throws java.io.IOException
	{
		try {
			stream.defaultReadObject();
			_keys= (Vector)stream.readObject();
		} catch (ClassNotFoundException cnfe) {
			System.err.println("OrderedHashtable.readObject: "
			+ "ClassNotFoundException " + cnfe);
			System.exit(1);
		}
	}

	/********************************************************************/
	// OrderedHashtable constructor : void ->
	/*********************************************************************
	** Constructs an empty ordered hashtable.
	*********************************************************************/
 
 	public OrderedHashtable () {
 		super();
	}
	
	/********************************************************************/
	// OrderedHashtable.put : Object x Object -> Object
	/*********************************************************************
	** Maps the specified key to the specified value in this hashtable.
	** Neither the key nor the value can be null. This method overrides
	** put in class Hashtable, and records the order in which keys are
	** put.
	** @param key the hashtable key.
	** @param value the value.
	** @return the previous value of the specified key in this hashtable,
	** or null if it did not have one.
	*********************************************************************/

	public Object put (Object key, Object value) {
		if (_keys != null && !_keys.contains(key)) {
			_keys.add(key);
		}
		return super.put(key,value);
	}
	
	/********************************************************************/
	// OrderedHashtable.remove : Object -> Object
	/*********************************************************************
	** Removes the key (and its corresponding value) from this hashtable.
	** This method does nothing if the key is not in the hashtable. This
	** method overrides remove in class Hashtable, deleting the key from
	** the list of keys.
	** @param key the key that needs to be removed.
	** @return the value to which the key had been mapped in this
	** hashtable, or null if the key did not have a mapping.
	*********************************************************************/

	public Object remove (Object key) {
		_keys.remove(key);
		return super.remove(key);
	}

	/********************************************************************/
	// OrderedHashtable.keyListIterator : void -> ListIterator
	/*********************************************************************
	** Returns an iterator over the keys in this hashtable, in the order
	** that they were put. (If a key was filled twice, its position in the
	** iterator is determined by the first time it was put.) This method
	** clones the list of keys to avoid the problems of concurrent
	** modification (the hashtable being modified while the keys are being
	** iterated over).
	** @return An iterator over the keys in this frame.
	*********************************************************************/

	public ListIterator keyListIterator () {
		return ((Vector)_keys.clone()).listIterator();
	}
}
