      ////////////////////////////////////////
     // THE CONSTRAINT-UNDERSTANDING SYSTEM //
    //  (Version 2)                        ///
   // Written by Andrew Broad             ////
  //                                     /////
 // Variable.java (1999:11:25 12:27)    //////
/////////////////////////////////////////////
// Methods of Variable:               //////
// - Variable constructor             /////
// - equals                           ////
// - toString                         ///
// - FILise                           //
///////////////////////////////////////

import java.io.*;

/*****************************************************************************
** The class `Variable' represents an occurrence of a variable in a case
** (a variable being an identifier that gets bound to a value when the case is 
** matched against a new situation).
*****************************************************************************/

public class Variable implements Serializable, FILisable {
	private String _name;

	/********************************************************************/
	// Variable constructor : String ->
	/*********************************************************************
	** Constructs an instance of Variable.
	** @param name The name of the variable (without the `?' prefix).
	*********************************************************************/

	public Variable (String name) {
		_name= name;
	}

	/********************************************************************/
	// Variable.equals : Object -> boolean
	/*********************************************************************
	** Returns true just in case the given object is a Variable whose name
	** is string-equal to this one.
	** @param anObject The Object to be compared.
	** @return True just in case the given object is an Variable whose
	**         name is string-equal to this one.
	*********************************************************************/

	public boolean equals (Object anObject) {
		return (anObject instanceof Variable
		       && _name.equals(((Variable)anObject).toString()));
	}

	/********************************************************************/
	// Variable.toString : void -> String
	/*********************************************************************
	** Returns the string representation of this variable, <I>without</I>
	** the `?' prefix.
	** @param anObject The string representation of this enumeration
	**  symbol.
	*********************************************************************/

	public String toString () {
		return _name;
	}

	/********************************************************************/
	// Variable.FILise : boolean -> String
	/*********************************************************************
	** Returns a string representing this variable in FIL.
	** @param HTML If true, the FIL will be augmented with HTML tags.
	** @return A string representing the variable in FIL.
	*********************************************************************/

	public String FILise (boolean HTML) {
		return "?" + _name;
	}
}